import os
import sys
from rdkit import Chem

from molvoxel.voxelizer.numpy import Voxelizer
from molvoxel.etc.rdkit.wrapper import MolWrapper, MolSystemWrapper, ComplexWrapper
from molvoxel.etc.rdkit.pointcloud import MolPointCloudMaker, MolSystemPointCloudMaker
from molvoxel.etc.rdkit.getter import AtomTypeGetter, BondTypeGetter

save_dir = 'result_rdkit'
if '-y' in sys.argv :
    pymol = True
    from molvoxel.etc.pymol import Visualizer
    visualizer = Visualizer()
    os.system(f'mkdir -p {save_dir}')
else :
    pymol = False
    visualizer = None

voxelizer = Voxelizer(dimension = 32)

""" LOAD DATA """
ligand_path = './10gs/10gs_ligand.sdf'
protein_path = './10gs/10gs_protein_nowater.pdb'

ligand_rdmol = Chem.SDMolSupplier(ligand_path)[0]
protein_rdmol = Chem.MolFromPDBFile(protein_path)

ligand_center = ligand_rdmol.GetConformer().GetPositions().mean(axis=0)

""" SINGLE MOL TEST """
rdmol = ligand_rdmol
center = ligand_center
atom_getter = AtomTypeGetter(['C', 'N', 'O', 'S'])
bond_getter = BondTypeGetter.default()

pointcloudmaker = MolPointCloudMaker(atom_getter, None, channel_type='types')
wrapper = MolWrapper(pointcloudmaker, voxelizer, visualizer)
image = wrapper.run(rdmol, center, radii=1.5)
if pymol :
    wrapper.visualize(f'{save_dir}/types.pse', ligand_rdmol, image, center)

pointcloudmaker = MolPointCloudMaker(atom_getter, bond_getter, channel_type='types')
wrapper = MolWrapper(pointcloudmaker, voxelizer, visualizer)
image = wrapper.run(rdmol, center, radii=1.5)
if pymol :
    wrapper.visualize(f'{save_dir}/bond_types.pse', ligand_rdmol, image, center)

pointcloudmaker = MolPointCloudMaker(atom_getter, None, channel_type='features')
wrapper = MolWrapper(pointcloudmaker, voxelizer, visualizer)
image = wrapper.run(rdmol, center, radii=1.5)
if pymol :
    wrapper.visualize(f'{save_dir}/features.pse', ligand_rdmol, image, center)

pointcloudmaker = MolPointCloudMaker(atom_getter, bond_getter, channel_type='features')
wrapper = MolWrapper(pointcloudmaker, voxelizer, visualizer)
image = wrapper.run(rdmol, center, radii=1.5)
if pymol :
    wrapper.visualize(f'{save_dir}/bond_features.pse', ligand_rdmol, image, center)

unknown_atom_getter = AtomTypeGetter(['C', 'N', 'O'], unknown=True)
pointcloudmaker = MolPointCloudMaker(unknown_atom_getter, bond_getter, channel_type='types')
wrapper = MolWrapper(pointcloudmaker, voxelizer, visualizer)
image = wrapper.run(rdmol, center, radii=1.5)
if pymol :
    wrapper.visualize(f'{save_dir}/unknownS.pse', ligand_rdmol, image, center)

""" SYSTEM TEST """
rdmol_list = [ligand_rdmol, protein_rdmol]
center = ligand_center
atom_getter = AtomTypeGetter(['C', 'N', 'O', 'S'])
bond_getter = BondTypeGetter.default()

pointcloudmaker = MolSystemPointCloudMaker([atom_getter, None], [atom_getter, bond_getter], channel_type='types')
wrapper = MolSystemWrapper(pointcloudmaker, voxelizer, ['Ligand', 'Protein'], visualizer)
image = wrapper.run(rdmol_list, center, radii=1.5)
if pymol :
    wrapper.visualize(f'{save_dir}/system.pse', [ligand_rdmol, protein_rdmol], image, center)

""" COMPLEX TEST """
pointcloudmaker = MolSystemPointCloudMaker([atom_getter, bond_getter], [atom_getter, None], channel_type='features')
wrapper = ComplexWrapper(pointcloudmaker, voxelizer, visualizer)
image = wrapper.run(ligand_rdmol, protein_rdmol, center, radii=1.5)
if pymol :
    wrapper.visualize(f'{save_dir}/complex.pse', ligand_rdmol, protein_rdmol, image, center)




from test_time_numpy import main

if __name__ == '__main__' :
    from molvoxel.voxelizer.numba import Voxelizer
    resolution = 0.5
    dimension = 48
    voxelizer = Voxelizer(resolution, dimension)
    main(voxelizer)



from test_time_numpy import main

if __name__ == '__main__' :
    from molvoxel.voxelizer.torch import Voxelizer
    import sys
    if '-g' in sys.argv :
        device = 'cuda'
    else :
        device = 'cpu'

    resolution = 0.5
    dimension = 48
    voxelizer = Voxelizer(resolution, dimension, device=device)
    main(voxelizer)



import sys
import os
import numpy as np
from rdkit import Chem

from utils import apply_coord

def main(Voxelizer, RandomTransform, pymol) :
    if pymol :
        from molvoxel.etc.pymol import Visualizer

    """ SET FUNCTION """
    def test(ligand_rdmol, protein_rdmol, atom_radii, save_dir) :
        if pymol :
            os.system(f'mkdir -p {save_dir}')
            visualizer = Visualizer()
        else :
            visualizer = None

        ligand_coords = ligand_rdmol.GetConformer().GetPositions()
        protein_coords = protein_rdmol.GetConformer().GetPositions()
        ligand_center = ligand_coords.mean(axis=0)
        center = ligand_center

        ligand_atom_radii = atom_radii[:ligand_rdmol.GetNumAtoms()]
        protein_atom_radii = atom_radii[ligand_rdmol.GetNumAtoms():]

        voxelizer = Voxelizer() #resolution=0.5, dimension=64, atom_scale=1.5, radii_type='scalar', density='gaussian'
        voxelizer_small = Voxelizer(0.5, 16, blockdim = 16)
        voxelizer_hr = Voxelizer(0.4, 64)

        transform = RandomTransform(random_translation=0.5, random_rotation=True)

        ligand_grid = voxelizer.get_empty_grid(1)
        protein_grid = voxelizer.get_empty_grid(1)

        print('Test 1: Radii Type: Scalar (Default), Density: Gaussian (Default)')
        test_name = 'ref'
        ligand_image = voxelizer.forward_single(ligand_coords, center, radii=1.0, out_grid=ligand_grid)
        protein_image = voxelizer.forward_single(protein_coords, center, radii=1.0, out_grid=protein_grid)
        assert ligand_image is ligand_grid, 'INPLACE FAILE'
        assert protein_image is protein_grid, 'INPLACE FAILE'
        if pymol :
            assert visualizer is not None
            visualizer.visualize_complex(f'{save_dir}/{test_name}.pse', ligand_rdmol, protein_rdmol, {'Atom': ligand_image.squeeze(0)}, {'Atom': protein_image.squeeze(0)}, center, resolution=voxelizer.resolution)

        print('Test 2: Small (One Block)')
        test_name = 'small'
        ligand_image = voxelizer_small.forward_single(ligand_coords, center, radii=1.0)
        protein_image = voxelizer_small.forward_single(protein_coords, center, radii=1.0)
        if pymol :
            assert visualizer is not None
            visualizer.visualize_complex(f'{save_dir}/{test_name}.pse', ligand_rdmol, protein_rdmol, {'Atom': ligand_image.squeeze(0)}, {'Atom': protein_image.squeeze(0)}, center, resolution=voxelizer.resolution)

        print('Test 3: High Resolution')
        test_name = 'hr'
        ligand_image = voxelizer_hr.forward_single(ligand_coords, center, radii=1.0, out_grid=ligand_grid)
        protein_image = voxelizer_hr.forward_single(protein_coords, center, radii=1.0, out_grid=protein_grid)
        assert ligand_image is ligand_grid, 'INPLACE FAILE'
        assert protein_image is protein_grid, 'INPLACE FAILE'
        if pymol :
            assert visualizer is not None
            visualizer.visualize_complex(f'{save_dir}/{test_name}.pse', ligand_rdmol, protein_rdmol, {'Atom': ligand_image.squeeze(0)}, {'Atom': protein_image.squeeze(0)}, center, resolution=voxelizer.resolution)

        print('Test 4: Radii Type: Atom-Wise')
        test_name = 'atom-wise'
        voxelizer.radii_type = 'atom-wise'
        ligand_image = voxelizer.forward_single(ligand_coords, center, radii=ligand_atom_radii, out_grid=ligand_grid)
        protein_image = voxelizer.forward_single(protein_coords, center, radii=protein_atom_radii, out_grid=protein_grid)
        assert ligand_image is ligand_grid, 'INPLACE FAILE'
        assert protein_image is protein_grid, 'INPLACE FAILE'
        if pymol :
            assert visualizer is not None
            visualizer.visualize_complex(f'{save_dir}/{test_name}.pse', ligand_rdmol, protein_rdmol, {'Atom': ligand_image.squeeze(0)}, {'Atom': protein_image.squeeze(0)}, center, resolution=voxelizer.resolution)

        print('Test 5: Density: Binary')
        test_name = 'binary'
        voxelizer.density = 'binary'
        voxelizer.radii_type = 'scalar'
        ligand_image = voxelizer.forward_single(ligand_coords, center, radii=1.0, out_grid=ligand_grid)
        protein_image = voxelizer.forward_single(protein_coords, center, radii=1.0, out_grid=protein_grid)
        assert ligand_image is ligand_grid, 'INPLACE FAILE'
        assert protein_image is protein_grid, 'INPLACE FAILE'
        if pymol :
            assert visualizer is not None
            visualizer.visualize_complex(f'{save_dir}/{test_name}.pse', ligand_rdmol, protein_rdmol, {'Atom': ligand_image.squeeze(0)}, {'Atom': protein_image.squeeze(0)}, center, resolution=voxelizer.resolution)

        print('Test 6: Random Transform')
        test_name = 'transform'
        voxelizer.density = 'gaussian'
        protein_coords = protein_rdmol.GetConformer().GetPositions()
        T = transform.get_transform()
        new_ligand_coords, new_protein_coords = T(ligand_coords, center), T(protein_coords, center)
        ligand_rdmol, protein_rdmol = apply_coord(ligand_rdmol, new_ligand_coords), apply_coord(protein_rdmol, new_protein_coords)
        ligand_image = voxelizer.forward_single(new_ligand_coords, center, radii=1.0, out_grid=ligand_grid)
        protein_image = voxelizer.forward_single(new_protein_coords, center, radii=1.0, out_grid=protein_grid)
        assert ligand_image is ligand_grid, 'INPLACE FAILE'
        assert protein_image is protein_grid, 'INPLACE FAILE'
        if pymol :
            assert visualizer is not None
            visualizer.visualize_complex(f'{save_dir}/{test_name}.pse', ligand_rdmol, protein_rdmol, {'Atom': ligand_image.squeeze(0)}, {'Atom': protein_image.squeeze(0)}, center, resolution=voxelizer.resolution)

    """ LOAD DATA """
    ligand_path = './10gs/10gs_ligand.sdf'
    protein_path = './10gs/10gs_protein_nowater.pdb'

    ligand_rdmol = Chem.SDMolSupplier(ligand_path)[0]
    protein_rdmol = Chem.MolFromPDBFile(protein_path)

    """ TEST """
    num_atoms = ligand_rdmol.GetNumAtoms() + protein_rdmol.GetNumAtoms()
    atom_radii = np.ones((num_atoms,))
    atom_radii[:ligand_rdmol.GetNumAtoms()] = 2.0

    save_dir = 'result_single'
    test(ligand_rdmol, protein_rdmol, atom_radii, save_dir)

if __name__ == '__main__' :
    if '-y' in sys.argv :
        pymol = True
    else :
        pymol = False

    from molvoxel.voxelizer.numpy import Voxelizer, RandomTransform
    main(Voxelizer, RandomTransform, pymol)



import sys
import os
from molvoxel.etc.rdkit.wrapper import ComplexWrapper
from molvoxel.etc.rdkit.pointcloud import MolSystemPointCloudMaker
from molvoxel.etc.rdkit.getter import AtomTypeGetter, BondTypeGetter, AtomFeatureGetter
import numpy as np
from rdkit import Chem

from utils import apply_coord

def main(Voxelizer, RandomTransform, pymol) :
    if pymol :
        from molvoxel.etc.pymol import Visualizer

    """ SET FUNCTION """
    def test(pointcloudmaker, ligand_rdmol, protein_rdmol, channel_radii, atom_radii, save_dir) :
        if pymol :
            os.system(f'mkdir -p {save_dir}')
            visualizer = Visualizer()
        else :
            visualizer = None

        ligand_coords = ligand_rdmol.GetConformer().GetPositions()
        ligand_center = ligand_coords.mean(axis=0)
        center = ligand_center

        voxelizer = Voxelizer() #resolution=0.5, dimension=64, atom_scale=1.5, radii_type='scalar', density='gaussian'
        voxelizer_small = Voxelizer(0.5, 16, blockdim = 16)
        voxelizer_hr = Voxelizer(0.4, 64)

        transform = RandomTransform(random_translation=0.5, random_rotation=True)

        wrapper = ComplexWrapper(pointcloudmaker, voxelizer, visualizer)
        wrapper_small = ComplexWrapper(pointcloudmaker, voxelizer_small, visualizer)
        wrapper_hr = ComplexWrapper(pointcloudmaker, voxelizer_hr, visualizer)

        grid = wrapper.get_empty_grid()

        print('Test 1: Radii Type: Scalar (Default), Density: Gaussian (Default)')
        test_name = 'ref'
        ref_image = wrapper.run(ligand_rdmol, protein_rdmol, center, radii=1.0)
        image = wrapper.run(ligand_rdmol, protein_rdmol, center, radii=1.0, out_grid = grid)
        assert image is grid, 'INPLACE FAILE'
        assert np.less(np.abs(np.subtract(image.tolist(), ref_image.tolist())), 1e-5).all(), 'REPRODUCTION FAIL'
        if pymol :
            wrapper.visualize(f'{save_dir}/{test_name}.pse', ligand_rdmol, protein_rdmol, image, center)

        print('Test 2: Small (One Block)')
        test_name = 'small'
        image = wrapper_small.run(ligand_rdmol, protein_rdmol, center, radii=1.0)
        if pymol :
            wrapper_small.visualize(f'{save_dir}/{test_name}.pse', ligand_rdmol, protein_rdmol, image, center)

        print('Test 3: High Resolution')
        test_name = 'hr'
        image = wrapper_hr.run(ligand_rdmol, protein_rdmol, center, radii=1.0)
        if pymol :
            wrapper_hr.visualize(f'{save_dir}/{test_name}.pse', ligand_rdmol, protein_rdmol, image, center)

        print('Test 4: Radii Type: Channel-Wise')
        test_name = 'channel-wise'
        voxelizer.radii_type = 'channel-wise'
        image = wrapper.run(ligand_rdmol, protein_rdmol, center, channel_radii, out_grid = grid)
        if pymol :
            wrapper.visualize(f'{save_dir}/{test_name}.pse', ligand_rdmol, protein_rdmol, image, center)

        print('Test 5: Radii Type: Atom-Wise')
        test_name = 'atom-wise'
        voxelizer.radii_type = 'atom-wise'
        image = wrapper.run(ligand_rdmol, protein_rdmol, center, atom_radii, out_grid = grid)
        if pymol :
            wrapper.visualize(f'{save_dir}/{test_name}.pse', ligand_rdmol, protein_rdmol, image, center)

        print('Test 6: Density: Binary')
        test_name = 'binary'
        voxelizer.density = 'binary'
        voxelizer.radii_type = 'scalar'
        image = wrapper.run(ligand_rdmol, protein_rdmol, center, radii=1.0, out_grid = grid)
        if pymol :
            wrapper.visualize(f'{save_dir}/{test_name}.pse', ligand_rdmol, protein_rdmol, image, center)

        print('Test 7: Random Transform')
        test_name = 'transform'
        voxelizer.density = 'gaussian'
        protein_coords = protein_rdmol.GetConformer().GetPositions()
        T = transform.get_transform()
        new_ligand_coords, new_protein_coords = T(ligand_coords, center), T(protein_coords, center)
        ligand_rdmol, protein_rdmol = apply_coord(ligand_rdmol, new_ligand_coords), apply_coord(protein_rdmol, new_protein_coords)
        image = wrapper.run(ligand_rdmol, protein_rdmol, center, radii=1.0, out_grid = grid)
        if pymol :
            wrapper.visualize(f'{save_dir}/{test_name}.pse', ligand_rdmol, protein_rdmol, image, center)

    """ LOAD DATA """
    ligand_path = './10gs/10gs_ligand.sdf'
    protein_path = './10gs/10gs_protein_nowater.pdb'

    ligand_rdmol = Chem.SDMolSupplier(ligand_path)[0]
    protein_rdmol = Chem.MolFromPDBFile(protein_path)

    """ Atom Types """
    print('# Test Atom Type #')
    save_dir = 'result_type'

    atom_getter = AtomTypeGetter(['C', 'N', 'O', 'S'])
    bond_getter = BondTypeGetter([Chem.BondType.SINGLE, Chem.BondType.DOUBLE, Chem.BondType.AROMATIC])
    pointcloudmaker = MolSystemPointCloudMaker([atom_getter, bond_getter], [atom_getter, bond_getter], channel_type='types')

    channel_radii = np.ones((pointcloudmaker.num_channels,))
    channel_radii[:4] = 2.0

    num_atoms = ligand_rdmol.GetNumAtoms() + ligand_rdmol.GetNumBonds() + protein_rdmol.GetNumAtoms() + protein_rdmol.GetNumBonds()
    atom_radii = np.ones((num_atoms,))
    atom_radii[:ligand_rdmol.GetNumAtoms()] = 2.0

    test(pointcloudmaker, ligand_rdmol, protein_rdmol, channel_radii, atom_radii, save_dir)

    """ Vector """
    print('# Test Atom Feature #')
    save_dir = 'result_feature'
    channels = ['C', 'N', 'O', 'S', 'Aromatic']
    def get_features(atom) :
        symbol_dict = {'C': 0, 'N': 1, 'O': 2, 'S': 3}
        res = [0] * 5
        symbol = atom.GetSymbol()
        res[symbol_dict[symbol]] = 1
        if atom.GetIsAromatic() : res[4] = 1
        return res

    atom_getter = AtomFeatureGetter(get_features, channels)
    bond_getter = BondTypeGetter([Chem.BondType.SINGLE, Chem.BondType.DOUBLE, Chem.BondType.AROMATIC])
    pointcloudmaker = MolSystemPointCloudMaker([atom_getter, None], [atom_getter, bond_getter], channel_type='features')

    channel_radii = np.ones((pointcloudmaker.num_channels,))
    channel_radii[[4, 9]] = 2.0

    num_atoms = ligand_rdmol.GetNumAtoms() + protein_rdmol.GetNumAtoms() + protein_rdmol.GetNumBonds()
    atom_radii = np.ones((num_atoms,))
    atom_radii[ -protein_rdmol.GetNumBonds() : ] = 2.0

    test(pointcloudmaker, ligand_rdmol, protein_rdmol, channel_radii, atom_radii, save_dir)

if __name__ == '__main__' :
    if '-y' in sys.argv :
        pymol = True
    else :
        pymol = False

    from molvoxel.voxelizer.numpy import Voxelizer, RandomTransform
    main(Voxelizer, RandomTransform, pymol)



import sys
from test_run_numpy_single import main

if __name__ == '__main__' :
    if '-y' in sys.argv :
        pymol = True
    else :
        pymol = False

    from molvoxel.voxelizer.numba import Voxelizer, RandomTransform
    main(Voxelizer, RandomTransform, pymol)



from setuptools import setup

with open('./README.md', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='molvoxel',
    version='0.1.2',
    description='MolVoxel:Easy-to-Use Molecular Voxelization Tool',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Seonghwan Seo',
    author_email='shwan0106@gmail.com',
    url='https://github.com/SeonghwanSeo/molvoxel',
    packages=['molvoxel/'],
    install_requires=['numpy', 'scipy'],
    extras_require={
            'numba': ['numba'],
            'torch': ['torch'],
            'rdkit': ['rdkit'],
    },
    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',

        'Development Status :: 4 - Beta',

        'Operating System :: OS Independent',

        'License :: OSI Approved :: MIT License',

        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Topic :: Scientific/Engineering :: Chemistry',
        'Topic :: Software Development :: Libraries :: Python Modules',

        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
    ],
)



import numpy as np
from rdkit import Chem
from molvoxel.etc.rdkit.wrapper import ComplexWrapper
from molvoxel.etc.rdkit.pointcloud import MolSystemPointCloudMaker
from molvoxel.etc.rdkit.getter import AtomTypeGetter, BondTypeGetter, AtomFeatureGetter
import time


def run_test(voxelizer, grid, coords, center, channels, radii, random_translation=0.5, random_rotation=True):
    batch_size = grid.shape[0]
    for i in range(batch_size):
        voxelizer.forward(coords, center, channels, radii, random_translation, random_rotation, out_grid=grid[i])
    return grid


def main(voxelizer):
    batch_size = 16
    num_iteration = 25
    num_trial = 5

    """ LOAD DATA """
    ligand_path = './10gs/10gs_ligand.sdf'
    protein_path = './10gs/10gs_protein_nowater.pdb'

    ligand_rdmol = Chem.SDMolSupplier(ligand_path)[0]
    protein_rdmol = Chem.MolFromPDBFile(protein_path)

    atom_getter = AtomTypeGetter(['C', 'N', 'O', 'S'])
    bond_getter = BondTypeGetter.default()
    pointcloudmaker_types = MolSystemPointCloudMaker([atom_getter, bond_getter], [atom_getter, bond_getter], channel_type='types')
    pointcloudmaker_features = MolSystemPointCloudMaker([atom_getter, bond_getter], [atom_getter, bond_getter], channel_type='features')
    wrapper_types = ComplexWrapper(pointcloudmaker_types, voxelizer)
    wrapper_features = ComplexWrapper(pointcloudmaker_features, voxelizer)

    ligand_coords = ligand_rdmol.GetConformer().GetPositions()
    ligand_center = ligand_coords.mean(0)
    center = ligand_center

    coords = wrapper_types.get_coords(ligand_rdmol, protein_rdmol)
    types = wrapper_types.get_channels(ligand_rdmol, protein_rdmol)
    features = wrapper_features.get_channels(ligand_rdmol, protein_rdmol)
    grid = wrapper_types.get_empty_grid(batch_size)
    center = voxelizer.asarray(center, 'center')

    single_grid = voxelizer.get_empty_grid(1, batch_size)

    """ SANITY CHECK """
    print('Sanity Check')
    for _ in range(2):
        single_out = run_test(voxelizer, single_grid, coords, center, None, 1.0, random_translation=0.0, random_rotation=False).tolist()
        type_out = run_test(voxelizer, grid, coords, center, types, 1.0, random_translation=0.0, random_rotation=False).tolist()
        feature_out = run_test(voxelizer, grid, coords, center, features, 1.0, random_translation=0.0, random_rotation=False).tolist()
        type_out = np.array(type_out)
        feature_out = np.array(feature_out)
        assert np.less(np.abs(type_out - type_out[0]), 1e-5).all(), 'REPRODUCTION FAIL'
        assert np.less(np.abs(feature_out - feature_out[0]), 1e-5).all(), 'REPRODUCTION FAIL'
        assert np.less(np.abs(type_out[0] - feature_out[0]), 1e-5).all(), 'REPRODUCTION FAIL'
    print('PASS\n')

    """ ATOM TYPE """
    print('Test Atom SINGLE')
    st_tot = time.time()
    for i in range(num_trial):
        print(f'trial {i}')
        st = time.time()
        for _ in range(num_iteration):
            single_grid = run_test(voxelizer, single_grid, coords, center, None, 1.0)
        end = time.time()
        print(f'total time {(end-st)}')
        print(f'time per run {(end-st) / batch_size / num_iteration}')
        print()
    end_tot = time.time()
    print(f'times per run {(end_tot-st_tot) / batch_size / num_iteration / num_trial}\n')

    """ ATOM TYPE """
    print('Test Atom Type')
    st_tot = time.time()
    for i in range(num_trial):
        print(f'trial {i}')
        st = time.time()
        for _ in range(num_iteration):
            grid = run_test(voxelizer, grid, coords, center, types, 1.0)
        end = time.time()
        print(f'total time {(end-st)}')
        print(f'time per run {(end-st) / batch_size / num_iteration}')
        print()
    end_tot = time.time()
    print(f'times per run {(end_tot-st_tot) / batch_size / num_iteration / num_trial}\n')

    """ ATOM FEATURE """
    print('Test Atom Feature')
    st_tot = time.time()
    for i in range(num_trial):
        print(f'trial {i}')
        st = time.time()
        for _ in range(num_iteration):
            grid = run_test(voxelizer, grid, coords, center, features, 1.0)
        end = time.time()
        print(f'total time {(end-st)}')
        print(f'time per run {(end-st) / batch_size / num_iteration}')
        print()
    end_tot = time.time()
    print(f'times per run {(end_tot-st_tot) / batch_size / num_iteration / num_trial}')


if __name__ == '__main__':
    from molvoxel.voxelizer.numpy import Voxelizer
    resolution = 0.5
    dimension = 48
    voxelizer = Voxelizer(resolution, dimension)
    main(voxelizer)
